from netgen.geom2d import unit_square
from ngsolve import *

mesh = Mesh(unit_square.GenerateMesh(maxh=0.1))

fes = VectorH1(mesh, order=2, dirichlet="left|right|top|bottom")
u = fes.TrialFunction()
v = fes.TestFunction()

#3d curl
#curl_u = CoefficientFunction( (grad(u)[2,1]-grad(u)[1,2], grad(u)[0,2]-grad(u)[2,0], grad(u)[1,0]-grad(u)[0,1]) )
#curl_v = CoefficientFunction( (grad(v)[2,1]-grad(u)[1,2], grad(v)[0,2]-grad(v)[2,0], grad(v)[1,0]-grad(v)[0,1]) )

#2d curl
curl_u = grad(u)[1,0]-grad(u)[0,1]
curl_v = grad(v)[1,0]-grad(v)[0,1]

# Solve equation -nabla div u + curl curl u + u = f
A = BilinearForm(fes)
A += SymbolicBFI( div(u)*div(v) + curl_u*curl_v + u*v )
A.Assemble()

f = LinearForm(fes)
f += SymbolicLFI( CoefficientFunction( (sin(x),cos(y)*x) )*v )
f.Assemble()

gfu = GridFunction(fes)
gfu.vec.data = A.mat.Inverse(fes.FreeDofs())*f.vec

Draw(gfu)
